#include <e.h>
#include "e_cfg.h"
#include "e_mod_win.h"

/* internal calls */
static Evas_Object *_e_cfg_win_new(const char *title, const char *name, const char *themedir, void (*delfunc) (const void *data), const void *data);
static void _e_cfg_win_complete(Evas_Object *ol);

static void _cb_signal_ok(void *data, Evas_Object *obj, const char *emission, const char *source);
static void _cb_delete(E_Win *win);
static void _cb_resize(E_Win *win);

static Evas_Object *_theme_obj_new(Evas *e, const char *custom_dir, const char *group);

/* state */
EAPI Illume_Cfg *illume_cfg = NULL;

static E_Config_DD *conf_edd = NULL;
static E_Module *mod = NULL;

/* called from the module core */
EAPI int
e_cfg_init(E_Module *m)
{
   mod = m;
   conf_edd = E_CONFIG_DD_NEW("Illume_Cfg", Illume_Cfg);
   
   E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.mode, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.icon_size, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, launcher.single_click, INT);
   
   E_CONFIG_VAL(conf_edd, Illume_Cfg, power.auto_suspend, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, power.auto_suspend_delay, INT);
   
   E_CONFIG_VAL(conf_edd, Illume_Cfg, performance.cache_level, INT);
   
   E_CONFIG_VAL(conf_edd, Illume_Cfg, slipshelf.main_gadget_size, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, slipshelf.extra_gagdet_size, INT);
   
   E_CONFIG_VAL(conf_edd, Illume_Cfg, sliding.slipshelf.duration, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, sliding.kbd.duration, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, sliding.busywin.duration, INT);
   E_CONFIG_VAL(conf_edd, Illume_Cfg, sliding.layout.duration, INT);

   illume_cfg = e_config_domain_load("module.illume", conf_edd);
   if (!illume_cfg)
     {
	illume_cfg = E_NEW(Illume_Cfg, 1);
	
	/* CONFIG DEFAULTS */
	illume_cfg->launcher.mode = 0;
	illume_cfg->launcher.icon_size = 120;
	illume_cfg->launcher.single_click = 1;
	
	illume_cfg->power.auto_suspend = 1;
	illume_cfg->power.auto_suspend_delay = 10;
	
	illume_cfg->performance.cache_level = 3;
	
	illume_cfg->slipshelf.main_gadget_size = 56;
	illume_cfg->slipshelf.extra_gagdet_size = 32;
	
	illume_cfg->sliding.slipshelf.duration = 1000;
	illume_cfg->sliding.kbd.duration = 1000;
	illume_cfg->sliding.busywin.duration = 1000;
	illume_cfg->sliding.layout.duration = 1000;
     }
   
   e_configure_registry_category_add("display", 0, "Display", NULL, "enlightenment/display");
   e_configure_registry_generic_item_add("display/launcher", 0, "Launcher", NULL, "enlightenment/launcher", e_cfg_launcher);
   e_configure_registry_generic_item_add("display/power", 0, "Power", NULL, "enlightenment/power", e_cfg_power);
   e_configure_registry_generic_item_add("display/animation", 0, "Animation", NULL, "enlightenment/animation", e_cfg_animation);
   return 1;
}

EAPI int
e_cfg_shutdown(void)
{
   e_configure_registry_item_del("display/animation");
   e_configure_registry_item_del("display/power");
   e_configure_registry_item_del("display/launcher");
   e_configure_registry_category_del("display");
   free(illume_cfg);
   E_CONFIG_DD_FREE(conf_edd);
   illume_cfg = NULL;
   mod = NULL;
   return 1;
}

EAPI int
e_cfg_save(void)
{
   e_config_domain_save("module.illume", conf_edd, illume_cfg);
   return 1;
}

///////////////////////////////////////////////////////////////////////////////
Ecore_Timer *_e_cfg_launcher_change_timer = NULL;
static int _e_cfg_launcher_change_timeout(void *data) {
   _e_mod_win_cfg_update();
   e_config_save_queue();
   _e_cfg_launcher_change_timer = NULL; return 0;
}
static void
_e_cfg_launcher_change(void *data, Evas_Object *obj, void *event_info) {
   if (_e_cfg_launcher_change_timer) ecore_timer_del(_e_cfg_launcher_change_timer);
   _e_cfg_launcher_change_timer = ecore_timer_add(0.5, _e_cfg_launcher_change_timeout, data);
}

EAPI void
e_cfg_launcher(E_Container *con, const char *params)
{
   Evas_Object *list, *o, *frame;
   E_Radio_Group *rg;
   Evas *e;

   list = _e_cfg_win_new("Launcher Settings", "launcher_settings",
			 e_module_dir_get(mod), NULL, NULL);
   
   e = evas_object_evas_get(list);
   
   frame = e_widget_framelist_add(e, "Display Type", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->launcher.mode));
   o = e_widget_radio_add(e, "Slider", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   o = e_widget_radio_add(e, "Icons", 1, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
   
   frame = e_widget_framelist_add(e, "Icon Size", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->launcher.icon_size));
   o = e_widget_radio_add(e, "Small", 60, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   o = e_widget_radio_add(e, "Medium", 80, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   o = e_widget_radio_add(e, "Large", 120, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   o = e_widget_radio_add(e, "Very large", 160, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   o = e_widget_radio_add(e, "Massive", 240, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
   
   frame = e_widget_framelist_add(e, "Launch Action", 0);
   o = e_widget_check_add(e, "Single press", &(illume_cfg->launcher.single_click));
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_launcher_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
   
   _e_cfg_win_complete(list);
}

///////////////////////////////////////////////////////////////////////////////
Ecore_Timer *_e_cfg_power_change_timer = NULL;
static int _e_cfg_power_change_timeout(void *data) {
   if (e_config->screensaver_timeout > 0)
     {
	e_config->screensaver_enable = 1;
     }
   else
     {
	e_config->screensaver_enable = 0;
	e_config->screensaver_timeout = 60;
     }
   if (illume_cfg->power.auto_suspend_delay > 0)
     {
	illume_cfg->power.auto_suspend = 1;
     }
   else
     {
	illume_cfg->power.auto_suspend = 0;
	illume_cfg->power.auto_suspend_delay = 30;
     }
   e_pwr_cfg_update();
   e_config_save_queue();
   _e_cfg_power_change_timer = NULL; return 0;
}
static void
_e_cfg_power_change(void *data, Evas_Object *obj, void *event_info) {
   if (_e_cfg_power_change_timer) ecore_timer_del(_e_cfg_power_change_timer);
   _e_cfg_power_change_timer = ecore_timer_add(0.5, _e_cfg_power_change_timeout, data);
}

EAPI void
e_cfg_power(E_Container *con, const char *params)
{
   Evas_Object *list, *o, *frame;
   E_Radio_Group *rg;
   Evas *e;

   list = _e_cfg_win_new("Power Settings", "power_settings",
			 e_module_dir_get(mod), NULL, NULL);
   
   e = evas_object_evas_get(list);
   
   frame = e_widget_framelist_add(e, "Blank Time", 0);
   rg = e_widget_radio_group_new(&(e_config->screensaver_timeout));
   o = e_widget_radio_add(e, "15 Seconds", 15, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "30 Seconds", 30, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "60 Seconds", 60, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "2 Minutes", 120, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "5 Minutes", 300, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
   
   frame = e_widget_framelist_add(e, "Suspend After Blank", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->power.auto_suspend_delay));
   o = e_widget_radio_add(e, "10 Seconds", 10, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "30 Seconds", 30, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "60 Seconds", 60, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_power_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
   
   _e_cfg_win_complete(list);
}

///////////////////////////////////////////////////////////////////////////////
Ecore_Timer *_e_cfg_animation_change_timer = NULL;
static int _e_cfg_animation_change_timeout(void *data) {
   e_config_save_queue();
   _e_cfg_animation_change_timer = NULL; return 0;
}
static void
_e_cfg_animation_change(void *data, Evas_Object *obj, void *event_info) {
   if (_e_cfg_animation_change_timer) ecore_timer_del(_e_cfg_animation_change_timer);
   _e_cfg_animation_change_timer = ecore_timer_add(0.5, _e_cfg_animation_change_timeout, data);
}

EAPI void
e_cfg_animation(E_Container *con, const char *params)
{
   Evas_Object *list, *o, *frame;
   E_Radio_Group *rg;
   Evas *e;

   list = _e_cfg_win_new("Animation Settings", "animation_settings",
			 e_module_dir_get(mod), NULL, NULL);
   
   e = evas_object_evas_get(list);
   
   frame = e_widget_framelist_add(e, "Applications", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->sliding.layout.duration));
   o = e_widget_radio_add(e, "Slow", 2000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Medium", 1000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Fast", 500, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Very Fast", 250, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
     
   frame = e_widget_framelist_add(e, "Top Shelf", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->sliding.slipshelf.duration));
   o = e_widget_radio_add(e, "Slow", 2000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Medium", 1000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Fast", 500, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Very Fast", 250, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
     
   frame = e_widget_framelist_add(e, "Keyboard", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->sliding.kbd.duration));
   o = e_widget_radio_add(e, "Slow", 2000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Medium", 1000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Fast", 500, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Very Fast", 250, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
     
   frame = e_widget_framelist_add(e, "Status", 0);
   rg = e_widget_radio_group_new(&(illume_cfg->sliding.busywin.duration));
   o = e_widget_radio_add(e, "Slow", 2000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Medium", 1000, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Fast", 500, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Very Fast", 250, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   o = e_widget_radio_add(e, "Off", 0, rg);
   e_widget_framelist_object_append(frame, o);
   evas_object_smart_callback_add(o, "changed", _e_cfg_animation_change, NULL);
   e_widget_list_object_append(list, frame, 1, 1, 0.0); // fill, expand, align
     
   _e_cfg_win_complete(list);
}


///////////////////////////////////////////////////////////////////////////////

/* internal calls */
typedef struct _Data Data;
struct _Data
{
   E_Win *win;
   Evas_Object *o, *o_sf, *o_l;
   void (*delfunc) (const void *data);
   const void *data;
};

static Evas_Object *
_e_cfg_win_new(const char *title, const char *name, const char *themedir, void (*delfunc) (const void *data), const void *data)
{
   E_Zone *zone;
   E_Win *win;
   Evas_Object *o, *sf, *ol;
   Data *d;
   
   zone = e_util_zone_current_get(e_manager_current_get());
   if (!zone) return NULL;
   win = e_win_new(zone->container);
   e_win_delete_callback_set(win, _cb_delete);
   e_win_resize_callback_set(win, _cb_resize);
   e_win_name_class_set(win, "E", name);
   e_win_title_set(win, title);
   d = E_NEW(Data, 1);
   d->win = win;
   d->delfunc = delfunc;
   d->data = data;
   win->data = d;
   o = _theme_obj_new(e_win_evas_get(win), themedir,
		      "e/modules/illume/config/dialog");
   edje_object_part_text_set(o, "e.text.label", "OK");
   edje_object_signal_callback_add(o, "e,action,do,ok", "", _cb_signal_ok, win);
   evas_object_move(o, 0, 0);
   evas_object_show(o);
   d->o = o;
   
   ol = e_widget_list_add(evas_object_evas_get(o), 0, 0);
   d->o_l = ol;
   
   evas_object_data_set(ol, "win", win);
   
   return ol;
}

static void
_e_cfg_win_complete(Evas_Object *ol)
{
   Evas_Object *sf;
   E_Win *win;
   Data *d;
   Evas_Coord mw, mh;

   win = evas_object_data_get(ol, "win");
   d = win->data;

   e_widget_min_size_get(ol, &mw, &mh);
   evas_object_resize(ol, mw, mh);
   
   sf = e_widget_scrollframe_simple_add(evas_object_evas_get(ol), ol);
   edje_object_part_swallow(d->o, "e.swallow.content", sf);
   d->o_sf = sf;
   
   e_win_show(win);

   e_widget_focus_set(ol, 1);
   evas_object_focus_set(ol, 1);
}

static void
_cb_signal_ok(void *data, Evas_Object *obj, const char *emission, const char *source)
{
   E_Win *win;
   Data *d;
   
   win = data;
   d = win->data;
   if (d->delfunc) d->delfunc(d->data);
   free(d);
   e_object_del(E_OBJECT(win));
}

static void
_cb_delete(E_Win *win)
{
   Data *d;
   
   d = win->data;
   if (d->delfunc) d->delfunc(d->data);
   free(d);
   e_object_del(E_OBJECT(win));
}
   
static void
_cb_resize(E_Win *win)
{
   Data *d;
   
   d = win->data;
   evas_object_resize(d->o, win->w, win->h);
}

static Evas_Object *
_theme_obj_new(Evas *e, const char *custom_dir, const char *group)
{
   Evas_Object *o;
   
   o = edje_object_add(e);
   if (!e_theme_edje_object_set(o, "base/theme/modules/illume", group))
     {
	if (custom_dir)
	  {
	     char buf[PATH_MAX];
	     
	     snprintf(buf, sizeof(buf), "%s/illume.edj", custom_dir);
	     if (edje_object_file_set(o, buf, group))
	       {
		  printf("OK FALLBACK %s\n", buf);
	       }
	  }
     }
   return o;
}
