#include <e.h>
#include "e_pwr.h"
#include "e_cfg.h"

/* internal calls */
static int _cb_saver(void *data, int ev_type, void *ev);

/* state */
static Ecore_Event_Handler *save_handler = NULL;
static Ecore_Timer *suspend_timer = NULL;
static Ecore_X_Window coverwin = 0;
static int suspended = 0;

/* called from the module core */
EAPI int
e_pwr_init(void)
{
   save_handler = ecore_event_handler_add(ECORE_X_EVENT_SCREENSAVER_NOTIFY,
					  _cb_saver, NULL);
   e_pwr_cfg_update();
   return 1;
}

EAPI int
e_pwr_shutdown(void)
{
   ecore_event_handler_del(save_handler);
   save_handler = NULL;
   return 1;
}

EAPI void
e_pwr_cfg_update(void)
{
   if (suspend_timer)
     {
	ecore_timer_del(suspend_timer);
	suspend_timer = NULL;
     }
   e_screensaver_init();
}

///////////////////////////////////////////////////////////////////////////////

/* internal calls */
static int
_cb_suspend(void *data)
{
   suspended = 1;
   
   /* FIXME: should be config */
   if (illume_cfg->power.auto_suspend)
     {
	printf("@@ SUSPEND NOW!\n");
//   system("/usr/lib/hal/scripts/linux/hal-system-power-suspend-linux");
	system("echo mem > /sys/power/state");
     }
   suspend_timer = NULL;
   return 0;
}

static int
_cb_saver(void *data, int ev_type, void *ev)
{
   Ecore_X_Event_Screensaver_Notify *event;
   
   event = ev;
   if (event->on)
     {
	if (!coverwin)
	  {
	     E_Zone *zone;
	     
	     zone = e_util_container_zone_number_get(0, 0);
	     if (zone)
	       {
		  printf("create coverwin\n");
		  coverwin = ecore_x_window_input_new(zone->container->win,
						      zone->x, zone->y,
						      zone->w, zone->h);
		  ecore_x_window_show(coverwin);
	       }
	  }
	printf("-------SAVER!!!!!!\n");
	if (suspend_timer)
	  {
	     ecore_timer_del(suspend_timer);
	     suspend_timer = NULL;
	  }
	if (illume_cfg->power.auto_suspend)
	  {
	     suspend_timer = ecore_timer_add(illume_cfg->power.auto_suspend_delay, _cb_suspend, NULL);
	  }
     }
   else
     {
	printf("-------WAKEUP!!!!!!\n");
	if (coverwin)
	  {
	     ecore_x_window_del(coverwin);
	     coverwin = 0;
	  }
	if (suspend_timer)
	  {
	     ecore_timer_del(suspend_timer);
	     suspend_timer = NULL;
	  }
 	if (suspended)
	  {
	     printf("@@ UNSUSPEND\n");
	     suspended = 0;
	  }
     }
   return 1;
}
